/*!
 * @file        main.c
 *
 * @brief       Main program body
 *
 * @version     V1.0.2
 *
 * @date        2023-03-31
 *
 * @attention
 *
 *  Copyright (C) 2023 Geehy Semiconductor
 *
 *  You may not use this file except in compliance with the
 *  GEEHY COPYRIGHT NOTICE (GEEHY SOFTWARE PACKAGE LICENSE).
 *
 *  The program is only for reference, which is distributed in the hope
 *  that it will be useful and instructional for customers to develop
 *  their software. Unless required by applicable law or agreed to in
 *  writing, the program is distributed on an "AS IS" BASIS, WITHOUT
 *  ANY WARRANTY OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the GEEHY SOFTWARE PACKAGE LICENSE for the governing permissions
 *  and limitations under the License.
 */

/* Includes */
#include "main.h"

/** @addtogroup Examples
  @{
*/

/** @addtogroup ADC_MultiChannelScan
  @{
*/

/** @defgroup ADC_MultiChannelScan_Macros Macros
  @{
*/

/* Debug USART definition */
#define DEBUG_USART                 USART1

/**@} end of group ADC_MultiChannelScan_Macros */

/** @defgroup ADC_MultiChannelScan_Functions Functions
  @{
*/

/* ADC Continuous Scan Init */
void ADC_ContinuousScanInit(void);
/* ADC conversion in polling */
void ADC_PollForConversion(void);
/* USART initialization */
void USART_Init(void);

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 */
int main(void)
{
    APM_MINI_LEDInit(LED2);
    APM_DelayInit(APM_DELAY_MS);
    USART_Init();

    /* ADC Continuous Scan Init */
    ADC_ContinuousScanInit();

    while(1)
    {
        ADC_PollForConversion();
        APM_MINI_LEDToggle(LED2);
        APM_Delay(500);
    }
}

/*!
 * @brief       ADC Calibration
 *
 * @param       None
 *
 * @retval      None
 */
void ADC_Calibration(void)
{
    uint8_t i;
    int8_t offset = 0;
    uint16_t adcData = 0;

    ADC_SetOffset(0);
    ADC_EnableCompensation();
    ADC_ClearStatusFlag(ADC_FLAG_CC);
    ADC_Enable();
    ADC_StartConversion();

    for(i = 0; i < 10; i++)
    {
        while(ADC_ReadStatusFlag(ADC_FLAG_CC) == RESET);
        ADC_ClearStatusFlag(ADC_FLAG_CC);
    }
    ADC_Disable();

    adcData = ADC_ReadData();
    offset = (int8_t)(0x800 - adcData);
    ADC_SetOffset(offset);

    ADC_DisableCompensation();
    ADC_ClearStatusFlag(ADC_FLAG_CC);
}

/*!
 * @brief       ADC continuous scan initialization
 *
 * @param       None
 *
 * @retval      None
 */
void ADC_ContinuousScanInit(void)
{
    ADC_Config_T adcConfig;
    GPIO_Config_T gpioConfig;

    /* ADC GPIO configuration */
    gpioConfig.intEn = GPIO_EINT_DISABLE;
    gpioConfig.mode = GPIO_MODE_IN_FLOATING;
    gpioConfig.pin = GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_4;
    GPIO_Config(GPIOC, &gpioConfig);

    ADC_SetMode(ADC_MODE_SINGLE_END);

    /* ADC configuration */
    ADC_ConfigStructInit(&adcConfig);
    adcConfig.div = ADC_DIV_4;
    adcConfig.convMode = ADC_CONV_MODE_CONTINUOUS;
    adcConfig.scanMode = ADC_SCAN_MODE_ENABLE;
    adcConfig.interrupt = ADC_INT_CC;
    adcConfig.channel = ADC_CHANNEL_2;
    ADC_Config(&adcConfig);

    /* ADC Calibration */
    ADC_Calibration();

    ADC_Enable();
    ADC_StartConversion();
}

/*!
 * @brief       ADC conversion in polling mode
 *
 * @param       None
 *
 * @retval      None
 */
void ADC_PollForConversion(void)
{
    uint16_t voltage = 0;
    uint16_t adcArray[10] = {0};
    uint8_t bufferIndex;

    if(ADC_ReadStatusFlag(ADC_FLAG_CC) == SET)
    {
        ADC_ClearStatusFlag(ADC_FLAG_CC);

        for(bufferIndex = 0; bufferIndex < 10; bufferIndex++)
        {
            adcArray[bufferIndex] = ADC_ReadBufferData((ADC_BUFFER_IDX_T)(bufferIndex));
        }

        for(bufferIndex = 0; bufferIndex < 3; bufferIndex++)
        {
            /* voltage(mV) =  adcData * (3300mV / 4095) */
            voltage = (adcArray[bufferIndex] * 3300) / 4095;
            printf("ADC Channel %d Value:%d\r\n", bufferIndex, voltage);
        }
    }
}

/*!
 * @brief       USART Initialization
 *
 * @param       None
 *
 * @retval      None
 */
void USART_Init(void)
{
    USART_Config_T usartConfig;

    /* 115200 baud rate */
    usartConfig.baudRate = 115200;
    /* No interrupt */
    usartConfig.interrupt = USART_INT_NONE;
    /* Enable transfer */
    usartConfig.mode = USART_MODE_TX;
    /* Parity disable */
    usartConfig.parity = USART_PARITY_NONE;
    /* One stop bit */
    usartConfig.stopBits = USART_STOP_BIT_1;
    /* 8bit word length */
    usartConfig.wordLength = USART_WORD_LEN_8B;

    APM_MINI_COMInit(COM1, &usartConfig);
}

#if defined (__CC_ARM) || defined (__ICCARM__) || (defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050))

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       ch:  The characters that need to be send.
*
* @param       *f:  pointer to a FILE that can recording all information
*              needed to control a stream
*
* @retval      The characters that need to be send.
*/
int fputc(int ch, FILE* f)
{
    /* send a byte of data to the serial port */
    USART_TxData8(DEBUG_USART, (uint8_t)ch);

    /* wait for the data to be send */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return (ch);
}

#elif defined (__GNUC__)

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       ch:  The characters that need to be send.
*
* @retval      The characters that need to be send.
*/
int __io_putchar(int ch)
{
    /* send a byte of data to the serial port */
    USART_TxData8(DEBUG_USART, ch);

    /* wait for the data to be send */
    while (USART_ReadStatusFlag(DEBUG_USART, USART_FLAG_TXBE) == RESET);

    return ch;
}

/*!
* @brief       Redirect C Library function printf to serial port.
*              After Redirection, you can use printf function.
*
* @param       file:  Meaningless in this function.
*
* @param       *ptr:  Buffer pointer for data to be sent.
*
* @param       len:  Length of data to be sent.
*
* @retval      The characters that need to be send.
*/
int _write(int file, char* ptr, int len)
{
    int i;
    for (i = 0; i < len; i++)
    {
        __io_putchar(*ptr++);
    }

    return len;
}

#else
#warning Not supported compiler type
#endif

/**@} end of group ADC_MultiChannelScan_Functions */
/**@} end of group ADC_MultiChannelScan */
/**@} end of group Examples */
